/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GLPointSet.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef GL_POINTSET_H
#define GL_POINTSET_H

#include "X3DGLGeometryNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// PointSet.

class PointSet : public X3DGeometryNode
{
public:
  /// Constructor.
  PointSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~PointSet();

  /// Sets the color attribute.
  void setColor(SFBool color);
  /// Sets the C4UB_V3F vertex array.
  void setC4UB_V3F_vertexArray(const std::vector<C4UB_V3F> &C4UB_V3F_vertexArray);
  /// Sets the V3F vertex array.  
  void setV3F_vertexArray(const MFVec3f &V3F_vertexArray);
  
  /// Gets the color attribute.
  inline SFBool getColor() const {return _color;};
  /// Gets the C4UB_V3F vertex array.
  inline std::vector<C4UB_V3F> &C4UB_V3F_vertexArray() {return _C4UB_V3F_vertexArray;};
  /// Sets the V3F vertex array.  
  inline MFVec3f &V3F_vertexArray() {return _V3F_vertexArray;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  
  
protected: 
  /// Copy constructor.	       
  PointSet(const PointSet &P);

private:
  /// Color attribute.
  SFBool _color;
  /// Point set array.
  std::vector<C4UB_V3F> _C4UB_V3F_vertexArray;
  /// Point set array.
  MFVec3f _V3F_vertexArray;
};

}
}

#endif
