/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Material.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef MATERIAL_H
#define MATERIAL_H

#include "X3DTypes.h"
#include "X3DMaterialNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining a Material
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#Material">
/// documentation</a>).

class Material : public X3DMaterialNode
{
public:
  /// Constructor.
  Material();
  /// Constructs a Material from its attributes.
  Material(SFFloat ambientIntensity, 
           const SFColor &diffuseColor, 
	   const SFColor &emissiveColor, 
	   SFFloat shininess, 
	   const SFColor &specularColor, 
	   SFFloat transparency);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Material();
  
  /// Sets the AmbientIntensity attribute.
  void setAmbientIntensity(SFFloat ambientIntensity);
  /// Sets the DiffuseColo attribute.
  void setDiffuseColor(const SFColor &diffuseColor);
  /// Sets the EmissiveColo attribute.
  void setEmissiveColor(const SFColor &emissiveColor);
  /// Sets the Shininess attribute.
  void setShininess(SFFloat shininess);  
  /// Sets the SpecularColor attribute.
  void setSpecularColor(const SFColor &specularColor);  
  /// Sets the Transparenc attribute.
  void setTransparency(SFFloat transparency);  
  
  /// Gets the AmbientIntensity attribute.
  inline SFFloat getAmbientIntensity() const {return _ambientIntensity;};       
  /// Gets the DiffuseColor attribute.
  inline SFColor getDiffuseColor() const {return _diffuseColor;};     
  /// Gets the EmissiveColor attribute.
  inline SFColor getEmissiveColor() const {return _emissiveColor; };    
  /// Gets the Shininess attribute.
  inline SFFloat getShininess() const {return _shininess;};        
  /// Gets the SpecularColor attribute.
  inline SFColor getSpecularColor() const {return _specularColor;};    
  /// Gets the Transparency attribute.
  inline SFFloat getTransparency() const {return _transparency;};   
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor. 
  Material(const Material &M);

private:    
  /// Ambient intensity.
  SFFloat _ambientIntensity;       
  /// Diffuse color.     
  SFColor _diffuseColor;     
  /// Emissive color.    
  SFColor _emissiveColor;    
  /// Shininess. 
  SFFloat _shininess;        
  /// Specular color. 
  SFColor _specularColor;    
  /// Transparency.
  SFFloat _transparency;     
};

}
}

#endif
