/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Shape.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef SHAPE_H
#define SHAPE_H

#include "X3DTypes.h"
#include "X3DShapeNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining a Shape
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#Shape">
/// documentation</a>).

class Shape : public X3DShapeNode, public X3DBoundedObject
{
public:
  /// Constructor.
  Shape();
  /// Constructs a Shape from its attributes.
  Shape(const X3DGeometryNode *geometry, 
        const X3DAppearanceNode *appearance, 
        const SFVec3f &bboxCenter, 
        const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Shape();
  
  /// loads the attributes of the node.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor. 	
  Shape(const Shape &S);
};

}
}

#endif
