/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DComponent.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENT_H
#define X3DCOMPONENT_H

#include "X3DTypes.h"
#include "Component.h"

namespace X3DTK {

/// Abstract class defining the concept of component.

class X3DComponent
{
public:
  /// Constructor.
  X3DComponent();
  /// Destructor.
  virtual ~X3DComponent() = 0;
  
  /// Returns the component.
  inline Component *getComponent() const {return component;};
  /// Returns the name of the component.
  inline SFString getName() const {return component->getName();};
  
  /// Adds one proxy.
  void addOneProxy();
  /// Removes one proxy.
  void removeOneProxy();
  
  /// Returns the number of proxies.
  inline unsigned int getProxyNumber() const {return _proxyNumber;};
  
protected:
  void defineComponentName(const SFString &name);
  Component *component;

private:  
  unsigned int _proxyNumber;  
};

}

#endif
