#include "X3DComponentVisitor.h"
#include "EnterFunction.h"
#include "WalkOnFunction.h"
#include "LeaveFunction.h"
#include "Type.h"

using namespace X3DTK;
using namespace std;

X3DComponentVisitor::X3DComponentVisitor()
{
}

X3DComponentVisitor::~X3DComponentVisitor()
{
  //delete the functions
  for (EnterDict::iterator it = enterDict_.begin(); it != enterDict_.end(); ++it)
    delete (*it).second; 
  
  for (WalkOnDict::iterator it = walkonDict_.begin(); it != walkonDict_.end(); ++it)
    delete (*it).second; 

  for (LeaveDict::iterator it = leaveDict_.begin(); it != leaveDict_.end(); ++it)
    delete (*it).second; 
}

void X3DComponentVisitor::setActiveAllFunctions()
{
  for (EnterDict::const_iterator it = enterDict_.begin(); it != enterDict_.end(); ++it)
    (*it).second->setActive();

  for (WalkOnDict::const_iterator it = walkonDict_.begin(); it != walkonDict_.end(); ++it)
    (*it).second->setActive();

  for (LeaveDict::const_iterator it = leaveDict_.begin(); it != leaveDict_.end(); ++it)
    (*it).second->setActive();
}

void X3DComponentVisitor::setInactiveAllFunctions()
{
  for (EnterDict::const_iterator it = enterDict_.begin(); it != enterDict_.end(); ++it)
    if ((*it).first != "X3DNode")
      (*it).second->setInactive();

  for (WalkOnDict::const_iterator it = walkonDict_.begin(); it != walkonDict_.end(); ++it)
    if ((*it).first != "X3DNode")
      (*it).second->setInactive();

  for (LeaveDict::const_iterator it = leaveDict_.begin(); it != leaveDict_.end(); ++it)
    if ((*it).first != "X3DNode")
      (*it).second->setInactive();
}  
  
EnterFunction *X3DComponentVisitor::getEnterFunctionOf(const Type *type) const
{
  SFString nodeName = type->getComponent()->getName() + "_" + type->getName();
  EnterDict::const_iterator it = enterDict_.find(nodeName);
  if ((it != enterDict_.end()) && ((*it).second->isActive()))
    return (*it).second;
    
  return 0;  
}

WalkOnFunction *X3DComponentVisitor::getWalkOnFunctionOf(const Type *type) const
{
  SFString nodeName = type->getComponent()->getName() + "_" + type->getName();
  WalkOnDict::const_iterator it = walkonDict_.find(nodeName);
  if ((it != walkonDict_.end()) && ((*it).second->isActive()))
    return (*it).second;
    
  return 0;  
}

LeaveFunction *X3DComponentVisitor::getLeaveFunctionOf(const Type *type) const
{ 
  SFString nodeName = type->getComponent()->getName() + "_" + type->getName();
  LeaveDict::const_iterator it = leaveDict_.find(nodeName);
  if ((it != leaveDict_.end()) && ((*it).second->isActive()))
    return (*it).second;
    
  //not found  
  return 0;  
}
