#include "GLTransform.h"
#include "Transform.h"
#include "GLScene.h"

using namespace std;

namespace X3DTK {
namespace GL {

Transform::Transform()
: X3DGroupingNode(), _transformMatrix(MFFloat(16))
{
  defineTypeName("Transform", "GLGrouping");
}

Transform::Transform(const Transform &T)
: X3DGroupingNode(T), _transformMatrix(T._transformMatrix) 
{
}

SFNode Transform::clone() const
{
  return new Transform(*this);
}

Transform::~Transform()
{
}

void Transform::setTransformMatrix(const MFFloat &transformMatrix)
{
  _transformMatrix = transformMatrix;
}

void Transform::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::Transform *T = static_cast<X3D::Transform *>(x3dReference);

  //computing the transformMatrix
  SFMatrix34f Tr = translation(T->getTranslation());
  SFMatrix34f C = translation(T->getCenter());
  SFVec3f v(T->getRotation().x, T->getRotation().y, T->getRotation().z);
  SFMatrix34f R = rotation(T->getRotation().angle, v);
  SFVec3f vs(T->getScaleOrientation().x, T->getScaleOrientation().y, T->getScaleOrientation().z);
  SFMatrix34f SR = rotation(T->getScaleOrientation().angle, vs);
  SFMatrix34f S = scale34(T->getScale().x, T->getScale().y, T->getScale().z);
  SFMatrix34f iSR = rotation(-T->getScaleOrientation().angle, vs);
  SFMatrix34f iC = translation(-1.0f*T->getCenter());
  SFMatrix34f transform = Tr*C*R*SR*S*iSR*iC;
   
  transform.toFloat16(_transformMatrix);
}

void Transform::render() const
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glMultMatrixf(&_transformMatrix.front());
}

}
}
