#include "ScalarInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

ScalarInterpolator::ScalarInterpolator()
  : X3DInterpolatorNode(),
    _keyValue(MFFloat())
{
  defineTypeName("ScalarInterpolator", "Interpolation");
}

ScalarInterpolator::ScalarInterpolator(const MFFloat &key,
                                       const MFFloat &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("ScalarInterpolator", "Interpolation");
}

ScalarInterpolator::ScalarInterpolator(const ScalarInterpolator &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFNode ScalarInterpolator::clone() const
{
  return new ScalarInterpolator(*this);
}

ScalarInterpolator::~ScalarInterpolator()
{
}

void ScalarInterpolator::setKeyValue(const MFFloat &keyValue)
{
  _keyValue = keyValue;
}

void ScalarInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFFloat(element->getAttribute(index));
}

SFString ScalarInterpolator::write() const
{
  SFString attr;
  if (!_keyValue.empty())
    attr += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return X3DInterpolatorNode::write() + attr;  
}

}
}
