#include "NurbsCurve2D.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsCurve2D::NurbsCurve2D()
  : X3DParametricGeometryNode(),
    _controlPoint(MFVec2f()),
    _tesselation (0),
    _weight      (MFDouble()),
    _knot        (MFDouble()),
    _order       (3)
{
  defineTypeName("NurbsCurve2D", "NURBS");
}

NurbsCurve2D::NurbsCurve2D(const MFVec2f  &controlPoint,
                           SFInt32        tesselation,
                           const MFDouble &weight,
                           const MFDouble &knot,
                           SFInt32        order)
  : X3DParametricGeometryNode(),
    _controlPoint(controlPoint),
    _tesselation (tesselation),
    _weight      (weight),
    _knot        (knot),
    _order       (order)
{
  defineTypeName("NurbsCurve2D", "NURBS");
}

NurbsCurve2D::NurbsCurve2D(const NurbsCurve2D &N)
  : X3DParametricGeometryNode(N),
    _controlPoint(N._controlPoint),
    _tesselation (N._tesselation),
    _weight      (N._weight),
    _knot        (N._knot),
    _order       (N._order)
{
}

SFNode NurbsCurve2D::clone() const
{
  return new NurbsCurve2D(*this);
}

NurbsCurve2D::~NurbsCurve2D()
{
}

void NurbsCurve2D::setControlPoint(const MFVec2f &controlPoint)
{
  _controlPoint = controlPoint;
}

void NurbsCurve2D::setTesselation(SFInt32 tesselation)
{
  _tesselation = tesselation;
}

void NurbsCurve2D::setWeight(const MFDouble &weight)
{
  _weight = weight;
}

void NurbsCurve2D::setKnot(const MFDouble &knot)
{
  _knot = knot;
}

void NurbsCurve2D::setOrder(SFInt32 order)
{
  _order = order;
}

void NurbsCurve2D::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("controlPoint");
  if (index != -1)
    _controlPoint = MFVec2f(element->getAttribute(index));

  index = element->getIndexAttribute("tesselation");
  if (index != -1)
    _tesselation = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("weight");
  if (index != -1)
    _weight = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("knot");
  if (index != -1)
    _knot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("order");
  if (index != -1)
    _order = element->getAttribute(index).toInt();
}

SFString NurbsCurve2D::write() const
{
  SFString attr;
  if (!_controlPoint.empty())
    attr += " controlPoint=\"" + toSFString(_controlPoint) + "\"";
  if (_tesselation != 0)
    attr += " tesselation=\"" + toSFString(_tesselation) + "\"";
  if (!_weight.empty())
    attr += " weight=\"" + toSFString(_weight) + "\"";
  if (!_knot.empty())
    attr += " knot=\"" + toSFString(_knot) + "\"";
  if (_order != 3)
    attr += " order=\"" + toSFString(_order) + "\"";

  return attr;
}


}
}
