/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            NurbsCurve2D.h                                  //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef NURBSCURVE2D
#define NURBSCURVE2D

#include "X3DTypes.h"
#include "X3DParametricGeometryNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/NURBS.html">
/// NURBS component</a> defining a NurbsCurve2D
/// (<a href="../../doc/X3DSpecs/Part01/components/NURBS.html#NurbsCurve2D">
/// documentation</a>).

class NurbsCurve2D : public X3DParametricGeometryNode
{
public:
  /// Constructor.
  NurbsCurve2D();
  /// Constructs a NurbsCurve2D from it attribute.
  NurbsCurve2D(const MFVec2f  &controlPoint,
               SFInt32        tesselation,
               const MFDouble &weight,
               const MFDouble &knot,
               SFInt32        order);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~NurbsCurve2D();

  /// Sets the controlPoint attribute.
  void setControlPoint(const MFVec2f &controlPoint);
  /// Sets the tesselation attribute.
  void setTesselation(SFInt32 tesselation);
  /// Sets the weight attribute.
  void setWeight(const MFDouble &weight);
  /// Sets the knot attribute.
  void setKnot(const MFDouble &knot);
  /// Sets the order attribute.
  void setOrder(SFInt32 order);

  /// Gets the controlPoint attribute.
  inline const MFVec2f &getControlPoint() const {return _controlPoint;};
  /// Gets the tesselation attribute.
  inline SFInt32 getTesselation() const {return _tesselation;};
  /// Gets the weight attribute.
  inline const MFDouble &getWeight() const {return _weight;};
  /// Gets the knot attribute.
  inline const MFDouble &getKnot() const {return _knot;};
  /// Gets the order attribute.
  inline SFInt32 getOrder() const {return _order;};

  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.  
  NurbsCurve2D(const NurbsCurve2D &N);

private:
  /// Set of control points.
  MFVec2f _controlPoint;
  /// Absolute number of subdivision steps.
  SFInt32 _tesselation;
  /// Weight values assigned to each control point.
  MFDouble _weight;
  /// Knot vector.
  MFDouble _knot;
  /// Order of the curve.
  SFInt32 _order;
};

}
}

#endif
