#include "NurbsPositionInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsPositionInterpolator::NurbsPositionInterpolator()
  : X3DInterpolatorNode(),
    _fractionAbsolute(true),
    _dimension       (0),
    _keyValue        (MFVec3f()),
    _keyWeight       (MFDouble()),
    _knot            (MFDouble()),
    _order           (3)
{
  defineTypeName("NurbsPositionInterpolator", "NURBS");
}

NurbsPositionInterpolator::NurbsPositionInterpolator(SFBool         fractionAbsolute,
                                                     SFInt32        dimension,
                                                     const MFFloat  &key,
                                                     const MFVec3f  &keyValue,
                                                     const MFDouble &keyWeight,
                                                     const MFDouble &knot,
                                                     SFInt32        order)
  : X3DInterpolatorNode(key),
    _fractionAbsolute(fractionAbsolute),
    _dimension       (dimension),
    _keyValue        (keyValue),
    _keyWeight       (keyWeight),
    _knot            (knot),
    _order           (order)
{
  defineTypeName("NurbsPositionInterpolator", "NURBS");
}

NurbsPositionInterpolator::NurbsPositionInterpolator(const NurbsPositionInterpolator &N)
  : X3DInterpolatorNode(N),
    _fractionAbsolute(N._fractionAbsolute),
    _dimension       (N._dimension),
    _keyValue        (N._keyValue),
    _keyWeight       (N._keyWeight),
    _knot            (N._knot),
    _order           (N._order)
{
}

SFNode NurbsPositionInterpolator::clone() const
{
  return new NurbsPositionInterpolator(*this);
}

NurbsPositionInterpolator::~NurbsPositionInterpolator()
{
}

void NurbsPositionInterpolator::setFractionAbsolute(SFBool fractionAbsolute)
{
  _fractionAbsolute = fractionAbsolute;
}

void NurbsPositionInterpolator::setDimension(SFInt32 dimension)
{
  _dimension = dimension;
}

void NurbsPositionInterpolator::setKeyValue(const MFVec3f &keyValue)
{
  _keyValue = keyValue;
}

void NurbsPositionInterpolator::setKeyWeight(const MFDouble &keyWeight)
{
  _keyWeight = keyWeight;
}

void NurbsPositionInterpolator::setKnot(const MFDouble &knot)
{
  _knot = knot;
}

void NurbsPositionInterpolator::setOrder(SFInt32 order)
{
  _order = order;
}

void NurbsPositionInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("fractionAbsolute");
  if (index != -1)
    _fractionAbsolute = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("dimension");
  if (index != -1)
    _dimension = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("keyWeight");
  if (index != -1)
    _keyWeight = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("knot");
  if (index != -1)
    _knot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("order");
  if (index != -1)
    _order = element->getAttribute(index).toInt();
}

SFString NurbsPositionInterpolator::write() const
{
  SFString attr;
  if (!_fractionAbsolute)
    attr += " fractionAbsolute=\"FALSE\"";
  if (_dimension != 0)
    attr += " dimension=\"" + toSFString(_dimension) + "\"";
  if (!_keyValue.empty())
    attr += " keyValue=\"" + toSFString(_keyValue) + "\"";
  if (!_keyWeight.empty())
    attr += " keyWeight=\"" + toSFString(_keyWeight) + "\"";
  if (!_knot.empty())
    attr += " knot=\"" + toSFString(_knot) + "\"";
  if (_order != 3)
    attr += " order=\"" + toSFString(_order) + "\"";

  return X3DInterpolatorNode::write() + attr;  
}

}
}
