#include "PointSet.h"
#include "X3DColorNode.h"
#include "X3DCoordinateNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

PointSet::PointSet()
: X3DGeometryNode(), _color(0), _coord(0)
{
  defineTypeName("PointSet", "Rendering");
}

PointSet::PointSet(const X3DColorNode *color, const X3DCoordinateNode *coord)
: X3DGeometryNode(), _color((SFNode)color), _coord((SFNode)coord)
{
  defineTypeName("PointSet", "Rendering");
}

PointSet::PointSet(const PointSet &P)
: X3DGeometryNode(P), _color(P._color), _coord(P._coord)
{
}

SFNode PointSet::clone() const
{
  return new PointSet(*this);
}

PointSet::~PointSet()
{
}

//modifiers
void PointSet::setColor(const X3DColorNode *color)
{
  removeLink(this, _color);
  _color = (SFNode)color;
  addLink(this, _color);
}

void PointSet::setCoord(const X3DCoordinateNode *coord)
{
  removeLink(this, _coord);
  _coord = (SFNode)coord;
  addLink(this, _coord);
}

bool PointSet::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (_color != 0)
      return false;
      
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (_coord != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "PointSet::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

bool PointSet::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "PointSet::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

bool PointSet::removeChild(const SFAbstractNode &N)
{
  if (_color == N)
  {
    setColor(0);
    return true;
  } 
   
  if (_coord == N)
  {
    setCoord(0);
    return true;
  } 
  
  return false;
}

}
}
