#include "TriangleSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TriangleSet::TriangleSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("TriangleSet", "Rendering");
}

TriangleSet::TriangleSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool convex, SFFloat creaseAngle, SFBool normalPerVertex, SFBool solid)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_convex(convex), _creaseAngle(creaseAngle)
{
  defineTypeName("TriangleSet", "Rendering");
}

TriangleSet::TriangleSet(const TriangleSet &T)
: X3DComposedGeometryNode(T), _convex(T._convex), _creaseAngle(T._creaseAngle)
{
}

SFNode TriangleSet::clone() const
{
  return new TriangleSet(*this);
}

TriangleSet::~TriangleSet()
{
}

void TriangleSet::setConvex(SFBool convex)
{
  _convex = convex;
}

void TriangleSet::setCreaseAngle(SFFloat creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void TriangleSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);

  int index;  
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
}    
    
SFString TriangleSet::write() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::write();
  
  if (!_convex)  
    attr += " convex=\"FALSE\"";
  if (_creaseAngle != 0.0f)
    attr += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
    
  return attr;
}

}
}
