#include "GLBuilderGeometry3DVisitor.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "IndexedFaceSet.h"
#include "Sphere.h"
#include "GLBox.h"
#include "GLCone.h"
#include "GLCylinder.h"
#include "GLIndexedFaceSet.h"
#include "GLSphere.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderGeometry3DVisitor::GLBuilderGeometry3DVisitor()
: Geometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLBuilderGeometry3DVisitor, Box>(&GLBuilderGeometry3DVisitor::enterBox);
  defineNewEnterFunction<GLBuilderGeometry3DVisitor, Cone>(&GLBuilderGeometry3DVisitor::enterCone);
  defineNewEnterFunction<GLBuilderGeometry3DVisitor, Cylinder>(&GLBuilderGeometry3DVisitor::enterCylinder);
  defineNewEnterFunction<GLBuilderGeometry3DVisitor, IndexedFaceSet>(&GLBuilderGeometry3DVisitor::enterIndexedFaceSet);
  defineNewEnterFunction<GLBuilderGeometry3DVisitor, Sphere>(&GLBuilderGeometry3DVisitor::enterSphere);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderGeometry3DVisitor::~GLBuilderGeometry3DVisitor()
{
}

void GLBuilderGeometry3DVisitor::enterBox(Box *B) const
{
  GL::Box *GB = new GL::Box();
  GB->setX3DReference(B);
  globalVariables->getTop()->addChild(GB);
  globalVariables->pushNode(GB);
}

void GLBuilderGeometry3DVisitor::enterCone(Cone *C) const
{
  GL::Cone *GC = new GL::Cone();
  GC->setX3DReference(C);
  globalVariables->getTop()->addChild(GC);
  globalVariables->pushNode(GC);
}

void GLBuilderGeometry3DVisitor::enterCylinder(Cylinder *C) const
{
  GL::Cylinder *GC = new GL::Cylinder();
  GC->setX3DReference(C);
  globalVariables->getTop()->addChild(GC);
  globalVariables->pushNode(GC);
}

void GLBuilderGeometry3DVisitor::enterIndexedFaceSet(IndexedFaceSet *I) const
{
  GL::IndexedFaceSet *GI = new GL::IndexedFaceSet();
  GI->setX3DReference(I);
  globalVariables->getTop()->addChild(GI);
  globalVariables->pushNode(GI);
}

void GLBuilderGeometry3DVisitor::enterSphere(Sphere *S) const
{
  GL::Sphere *GS = new GL::Sphere();
  GS->setX3DReference(S);
  globalVariables->getTop()->addChild(GS);
  globalVariables->pushNode(GS);
}

}
}
