#include "SimpleX3DGLScene.h"
#include "DefaultLoader.h"
#include "BboxUpdater.h"
#include "GLBuilder.h"
#include "GLRenderer.h"
#include "MemReleaser.h"
#include "X3DFileValidator.h"
#include "Scene.h"
#include "GLScene.h"
#include "Bbox.h"

#include <iostream>
#include <time.h>

using namespace std;

namespace X3DTK {

SimpleX3DGLScene::SimpleX3DGLScene()
: scene(0), glscene(0)
{
  _loader = new DefaultLoader();
  _bboxupdater = new X3D::BboxUpdater();
  _builder = new X3D::GLBuilder();
  _renderer = new GL::Renderer();
  _releaser = new MemReleaser();
}

SimpleX3DGLScene::~SimpleX3DGLScene()
{
  _releaser->release(scene);
  _releaser->release(glscene);  
  scene = 0;
  glscene = 0;

  delete _loader;
  delete _bboxupdater;
  delete _builder;
  delete _renderer;
  delete _releaser;
}

void SimpleX3DGLScene::setLoader(X3DLoader *loader)
{
  _loader = loader;
}

void SimpleX3DGLScene::setBboxUpdater(X3D::X3DBboxUpdater *bboxUpdater)
{
  _bboxupdater = bboxUpdater;
}

void SimpleX3DGLScene::setGLBuilder(X3D::X3DGLBuilder *builder)
{
  _builder = builder;
}

void SimpleX3DGLScene::setGLRenderer(GL::X3DRenderer *renderer)
{
  _renderer = renderer;
}

void SimpleX3DGLScene::setMemReleaser(X3DMemReleaser *memReleaser)
{
  _releaser = memReleaser;
}
  
void SimpleX3DGLScene::setFileValidator(X3D::X3DFileValidator *fileValidator)
{
  _loader->setFileValidator(fileValidator);
}
  
void SimpleX3DGLScene::load(const char *file, bool fileValidation)
{
  loadFile(file, fileValidation);
  computeBbox();
  buildGLScene();
}

void SimpleX3DGLScene::draw()
{
  _renderer->render(glscene);
}

void SimpleX3DGLScene::release()
{
  _releaser->release(scene);
  _releaser->release(glscene);
  scene = 0;
  glscene = 0;  
}

void SimpleX3DGLScene::loadFile(const char *file, bool fileValidation)
{
  _releaser->release(scene);
  _releaser->release(glscene);  
  scene = 0;
  glscene = 0;
  
  scene = _loader->load(file, fileValidation);
}

void SimpleX3DGLScene::computeBbox()
{
  _bboxupdater->update(scene, true);
}

void SimpleX3DGLScene::buildGLScene()
{
  if (scene != 0)
  {
    SFVec3f center = scene->getBboxCenter();
    SFVec3f size = scene->getBboxSize();

    SFVec3f A = center + 0.5f*size;
    SFVec3f B = center - 0.5f*size;

    if (A.x < B.x)
    {
      min.x = A.x;
      max.x = B.x;
    }  
    else
    {
      min.x = B.x;  
      max.x = A.x;
    }  
  
    if (A.y < B.y)
    {
      min.y = A.y;
      max.y = B.y;
    }  
    else
    {
      min.y = B.y;  
      max.y = A.y;
    }
  
    if (A.z < B.z)
    {
      min.z = A.z;
      max.z = B.z;
    }  
    else
    {
      min.z = B.z;  
      max.z = A.z;
    }  
   
    glscene = _builder->build(scene);
  }
}


}
