#include "WorldCoordTranslatorGroupingVisitor.h"
#include "Group.h"
#include "Transform.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

WorldCoordTranslatorGroupingVisitor::WorldCoordTranslatorGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<WorldCoordTranslatorGroupingVisitor, Transform>(&WorldCoordTranslatorGroupingVisitor::enterTransform);
  
  // Leave functions.
  defineNewLeaveFunction<WorldCoordTranslatorGroupingVisitor, Transform>(&WorldCoordTranslatorGroupingVisitor::leaveTransform);
  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<WorldCoordTranslatorGlobalVariables>();
}

WorldCoordTranslatorGroupingVisitor::~WorldCoordTranslatorGroupingVisitor()
{
}

void WorldCoordTranslatorGroupingVisitor::enterTransform(Transform *T) const
{
  //change the coordinates of the Bbox in the coordinates of the parent node
  SFMatrix34f Tr = translation(T->getTranslation());
  SFMatrix34f C = translation(T->getCenter());
  SFVec3f v(T->getRotation().x, T->getRotation().y, T->getRotation().z);
  SFMatrix34f R = rotation(T->getRotation().angle, v);
  SFVec3f vs(T->getScaleOrientation().x, T->getScaleOrientation().y, T->getScaleOrientation().z);
  SFMatrix34f SR = rotation(T->getScaleOrientation().angle, vs);
  SFMatrix34f S = scale34(T->getScale().x, T->getScale().y, T->getScale().z);
  SFMatrix34f iSR = rotation(-T->getScaleOrientation().angle, vs);
  SFMatrix34f iC = translation(-1.0f*T->getCenter());
  
  SFMatrix34f transform = Tr*C*R*SR*S*iSR*iC;
  
  T->setCenter(SFVec3f(0.0f, 0.0f, 0.0f));
  T->setRotation(SFRotation(0.0f, 0.0f, 1.0f, 0.0f));
  T->setScale(SFVec3f(1.0f, 1.0f, 1.0f));
  T->setScaleOrientation(SFRotation(0.0f, 0.0f, 1.0f, 0.0f));
  T->setTranslation(SFVec3f(0.0f, 0.0f, 0.0f));
  
  globalVariables->pushMatrix(transform);
}

void WorldCoordTranslatorGroupingVisitor::leaveTransform(Transform *T) const
{
  globalVariables->popMatrix();
}

}
}
